using MWS.Library.Common.Module;
using MWS.Model;
using MWS.Model.Dto;
using System;
using System.Collections.Generic;

namespace MWS.Data.InterFace
{
    public interface IWeight : IDapperHelper
    {
        Record GetOneByDevice(int deviceId);
        List<Record> GetChartSource(DateTime? startTime, DateTime? endTime);

        List<RecordDto> GetList( DateTime? startTime, DateTime? endTime);
        int Delete(int id);
        TotalDto TotalStatistics();
        List<DailyStatisticDto> DailyStatistics(int teamId, int deviceId);

        List<DailyStatisticDto> DailyStatistics(int deviceId);

        List<DailyStatisticDto> DailyTotalStatistics(int teamId);

        List<RealTimeBarDto> GetDailyRealTimeData(int deviceId, int teamId);
        int Insert(Record t_Record);
        object GetTotal(int deviceId = 0, int teamId = 0,int entranceId=0);
        PageData<RecordDto> GetAllRecords(int entranceId, int deviceId, int teamId, DateTime? beginTime, DateTime? endTime, int pageIndex, int pageSize);
        List<RecordDto> GetAllExportRecords(int entranceId, int deviceId, int teamId, DateTime? beginTime, DateTime? endTime);
        int GetAllRecordsCount(int entranceId, int deviceId, int teamId, DateTime? beginTime, DateTime? endTime);
        List<RecordDto> GetChartRecords(int entranceId, int deviceId, int teamId, DateTime? beginTime, DateTime? endTime);
        Record GetNextOne(int id,int deviceId,int isNext);
    }
}
